using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Net;

namespace pipa
{
    /// <summary>
    /// The main menu screen is the first thing displayed when the game starts up.
    /// </summary>
    internal class MainMenuScreen : MenuScreen
    {
        private Texture2D mainMenuTexture;
        private Rectangle screenRect;

        public MainMenuScreen()
            : base(StringResources.Menu)
        {
            
        }

        public override void LoadContent()
        {
            SpriteFont font = ScreenManager.Font;
            GraphicsDeviceManager graphics = ScreenManager.Graphics;
            int screenWidth = graphics.PreferredBackBufferWidth;

            // Create our menu entries.
            Vector2 size = font.MeasureString(StringResources.StartGame);
            MenuEntry singlePlayerMenuEntry = new MenuEntry(StringResources.StartGame, (int)size.X, (int)size.Y);
            singlePlayerMenuEntry.Position = new Vector2(screenWidth / 2 - size.X / 2, 420);

            size = font.MeasureString(StringResources.Exit);
            MenuEntry exitMenuEntry = new MenuEntry(StringResources.Exit, (int)size.X, (int)size.Y);
            exitMenuEntry.Position = new Vector2(screenWidth / 2 - size.X / 2, 720);
            exitMenuEntry.Scale = 0.5f;

            // Hook up menu event handlers.
            singlePlayerMenuEntry.Selected += SinglePlayerMenuEntrySelected;
            exitMenuEntry.Selected += OnCancel;

            // Add entries to the menu.
            MenuEntries.Add(singlePlayerMenuEntry);
            MenuEntries.Add(exitMenuEntry);

            mainMenuTexture = ScreenManager.Content.Load<Texture2D>("menu/ui_mainmenu");
            screenRect = new Rectangle(0, 0, ScreenManager.Graphics.PreferredBackBufferWidth,
                                       ScreenManager.Graphics.PreferredBackBufferHeight);
            base.LoadContent();
        }

        private void SinglePlayerMenuEntrySelected(object sender, EventArgs args)
        {
            LoadingScreen.Load(ScreenManager,
                               new GameplayScreen());
        }

        public override void Draw(GameTime gameTime)
        {
            ScreenManager.SpriteBatch.Begin();

            ScreenManager.SpriteBatch.Draw(mainMenuTexture, screenRect, Color.White);

            ScreenManager.SpriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// When the user cancels the main menu, ask if they want to exit the sample.
        /// </summary>
        protected override void OnCancel(object o, EventArgs args)
        {
            MessageBoxScreen confirmExitMessageBox = new MessageBoxScreen(StringResources.AreYouSure);

            confirmExitMessageBox.Accepted += ConfirmExitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmExitMessageBox);
        }


        /// <summary>
        /// Event handler for when the user selects ok on the "are you sure
        /// you want to exit" message box.
        /// </summary>
        private void ConfirmExitMessageBoxAccepted(object sender, object f)
        {
            ScreenManager.Game.Exit();
        }
    }
}