#region Using Statements

using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Net;

#endregion

namespace pipa
{
    internal class PauseMenuScreen : MenuScreen
    {
        public PauseMenuScreen()
            : base(StringResources.Paused)
        {
            
        }

        public override void LoadContent()
        {
            IsPopup = true;

            SpriteFont font = ScreenManager.Font;
            GraphicsDeviceManager graphics = ScreenManager.Graphics;
            int screenWidth = graphics.PreferredBackBufferWidth;

            // Add the Resume Game menu entry.
            Vector2 size = font.MeasureString(StringResources.Resume);
            MenuEntry resumeGameMenuEntry = new MenuEntry(StringResources.Resume, (int)size.X, (int)size.Y);
            resumeGameMenuEntry.Position = new Vector2(screenWidth / 2 - size.X / 2, 300);
            resumeGameMenuEntry.Selected += OnCancel;
            MenuEntries.Add(resumeGameMenuEntry);

            // If this is a single player game, add the Quit menu entry.
            size = font.MeasureString(StringResources.QuitGame);
            MenuEntry quitGameMenuEntry = new MenuEntry(StringResources.QuitGame, (int)size.X, (int)size.Y);
            quitGameMenuEntry.Position = new Vector2(screenWidth / 2 - size.X / 2, 300 + resumeGameMenuEntry.Height + 25);
            quitGameMenuEntry.Selected += QuitGameMenuEntrySelected;
            MenuEntries.Add(quitGameMenuEntry);

            base.LoadContent();
        }

        private void QuitGameMenuEntrySelected(object sender, EventArgs args)
        {
            MessageBoxScreen confirmQuitMessageBox =
                new MessageBoxScreen(StringResources.AreYouSure);

            confirmQuitMessageBox.Accepted += ConfirmQuitMessageBoxAccepted;

            ScreenManager.AddScreen(confirmQuitMessageBox);
        }

        private void ConfirmQuitMessageBoxAccepted(object sender, EventArgs args)
        {
            LoadingScreen.Load(ScreenManager, null, new BackgroundScreen(),
                               new MainMenuScreen());
        }
        
        public override void Draw(GameTime gameTime)
        {
            ScreenManager.FadeBackBufferToBlack(TransitionAlpha*2/3);
            base.Draw(gameTime);
        }
    }
}