﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace pipa.Scene
{
    public class Rope
    {
        Texture2D pixel;
        List<Vector2> vectors;

        /// <summary>
        /// Gets/sets the colour of the primitive line object.
        /// </summary>
        public Color Colour;

        /// <summary>
        /// Gets/sets the position of the primitive line object.
        /// </summary>
        public Vector2 Position;

        /// <summary>
        /// Gets/sets the render depth of the primitive line object (0 = front, 1 = back)
        /// </summary>
        public float Depth;

        /// <summary>
        /// Gets the number of vectors which make up the primtive line object.
        /// </summary>
        public int CountVectors
        {
            get
            {
                return vectors.Count;
            }
        }

        // initiate the Chainvektor between the players
        private Vector2[] PlayerChain;

        /// <summary>
        /// Creates a new primitive line object.
        /// </summary>
        /// <param name="graphicsDevice">The Graphics Device object to use.</param>
        public Rope(ScreenManager SCM_Name)
        {
            // create pixels
            pixel = SCM_Name.Content.Load<Texture2D>("scene/players/ch_chain");

            Colour = Color.White;
            Position = new Vector2(0, 0);
            Depth = 0;

            vectors = new List<Vector2>();

            PlayerChain = new Vector2[25];


        }

        /// <summary>
        /// Adds a vector to the primive live object.
        /// </summary>
        /// <param name="vector">The vector to add.</param>
        public void AddVector(Vector2 vector)
        {
            vectors.Add(vector);
        }

        /// <summary>
        /// Insers a vector into the primitive line object.
        /// </summary>
        /// <param name="index">The index to insert it at.</param>
        /// <param name="vector">The vector to insert.</param>
        public void InsertVector(int index, Vector2 vector)
        {
            vectors.Insert(index, vector);
        }

        /// <summary>
        /// Removes a vector from the primitive line object.
        /// </summary>
        /// <param name="vector">The vector to remove.</param>
        public void RemoveVector(Vector2 vector)
        {
            vectors.Remove(vector);
        }

        /// <summary>
        /// Removes a vector from the primitive line object.
        /// </summary>
        /// <param name="index">The index of the vector to remove.</param>
        public void RemoveVector(int index)
        {
            vectors.RemoveAt(index);
        }

        /// <summary>
        /// Clears all vectors from the primitive line object.
        /// </summary>
        public void ClearVectors()
        {
            vectors.Clear();
        }

        /// <summary>
        /// Renders the primtive line object.
        /// </summary>
        /// <param name="spriteBatch">The sprite batch to use to render the primitive line object.</param>
        public void Render(SpriteBatch spriteBatch)
        {
            for (int index = 0; index < 25; index++)
            {
                spriteBatch.Draw(pixel, PlayerChain[index], Color.White);
            }            
        }

        public void get_ChainVector(Vector2 PosPlayer1, Vector2 PosPlayer2)
        {
            float p, q;
            float dx;
            int ChainElemets;
            ChainElemets = 25;
            dx = Math.Abs(PosPlayer1.X - PosPlayer2.X) / ChainElemets;
            float offset_dx;
            float offset_dy;
            if (PosPlayer1.X < PosPlayer2.X)
            {
                offset_dx = PosPlayer1.X;
            }
            else
            {
                offset_dx = PosPlayer2.X;
            }          
            
            // linearisation for disances less than 30 
            /**/
            q = PosPlayer1.Y - PosPlayer1.X *( (PosPlayer2.Y - PosPlayer1.Y) / (PosPlayer2.X - PosPlayer1.X) );
            p = (PosPlayer2.Y - PosPlayer1.Y) / (PosPlayer2.X - PosPlayer1.X); 
            for (int index = 0; index < ChainElemets; index++)
            {
                PlayerChain[index].X = offset_dx + (index * dx);
                if (q > 0)
                {
                    if (PosPlayer1.X < PosPlayer2.X)
                    {
                        offset_dy = PosPlayer1.Y;
                    }
                    else
                    {
                        offset_dy = PosPlayer2.Y;
                    }
                }
                else
                {
                    if (PosPlayer1.X < PosPlayer2.X)
                    {
                        offset_dy = PosPlayer1.Y;
                    }
                    else
                    {
                        offset_dy = PosPlayer2.Y;
                    }
                }
                PlayerChain[index].Y = (index * dx * p + offset_dy);
            }

            /*/

            p = ( PosPlayer2.Y - PosPlayer1.Y - PosPlayer2.X * PosPlayer2.X + PosPlayer1.X * PosPlayer1.X) /
                (PosPlayer2.X - PosPlayer1.X);
            q = PosPlayer1.Y - PosPlayer1.X * PosPlayer1.X - PosPlayer1.X * p;

            if (PosPlayer1.X < PosPlayer2.X)
            {
                offset_dx = PosPlayer1.X;
            }
            else
            {
                offset_dx = PosPlayer2.X;
            }

            if (PosPlayer1.X < PosPlayer2.X)
            {
                offset_dy = PosPlayer1.Y;
            }
            else
            {
                offset_dy = PosPlayer2.Y;
            }
            for (int index = 0; index < ChainElemets; index++)
            {
                PlayerChain[index].X = (index * dx ) + offset_dx;
                PlayerChain[index].Y = (index * dx * index * dx + index * dx * p - offset_dy);
            }
            /**/
        }
    }
}