﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace pipa.Scene
{
    public class Player
    {
        // Animations
        private Animation idleAnimation;
        private Animation runAnimation;
        private Animation jumpAnimation;
        private Animation jumprunAnimation;
        // private Animation celebrateAnimation;
        // private Animation dieAnimation;
        private SpriteEffects flip = SpriteEffects.None;
        private AnimationPlayer sprite;
        private Player otherPlayer;

        // keyset dependent on playerident 
        // graph depend on playerident
        public int PlayerIdent;
        public bool CollisionVictim;
        public bool CollisionReason;

        // Sounds
        // private SoundEffect killedSound;
        private SoundEffect jumpSound;
        private SoundEffect fallSound;
        private SoundEffect gegnerSchiebenSound;
        private SoundEffect gegnerZiehenSound;
        private SoundEffect landeAufGegnerSound;

        public Level Level
        {
            get { return level; }
        }

        private Level level;

        public bool IsAlive
        {
            get { return isAlive; }
        }

        private bool isAlive;

        // Physics state
        public Vector2 Position
        {
            get { return position; }
            set { position = value; }
        }

        private Vector2 position;

        private float previousBottom;

        public Vector2 Velocity
        {
            get { return velocity; }
            set { velocity = value; }
        }

        private Vector2 velocity;

        // Constants for controling horizontal movement
        private const float MoveAcceleration = 10000.0f;
        private const float MaxMoveSpeed = 250.0f;
        private const float GroundDragFactor = 0.45f;
        private const float AirDragFactor = 0.65f;

        // Constants for controlling vertical movement
        private const float MaxJumpTime = 0.60f;
        private const float JumpLaunchVelocity = -4000.0f;
        private const float GravityAcceleration = 3500.0f;
        private const float MaxFallSpeed = 1000.0f;
        private const float JumpControlPower = 0.14f;

        // Input configuration
        private const float MoveStickScale = 1.0f;
        private const Buttons JumpButton = Buttons.A;

        /// <summary>
        /// Gets whether or not the player's feet are on the ground.
        /// </summary>
        public bool IsOnGround
        {
            get { return isOnGround; }
        }

        private bool isOnGround;

        /// <summary>
        /// Current user movement input.
        /// </summary>
        private float movement;

        // Jumping state
        private bool isJumping;
        private bool wasJumping;
        private float jumpTime;

        private bool isLockedByChain;

        private Rectangle localBounds;

        /// <summary>
        /// Gets a rectangle which bounds this player in world space.
        /// </summary>
        public Rectangle BoundingRectangle
        {
            get
            {
                int left = (int) Math.Round(Position.X - sprite.Origin.X) + localBounds.X;
                int top = (int) Math.Round(Position.Y - sprite.Origin.Y) + localBounds.Y;

                return new Rectangle(left, top, localBounds.Width, localBounds.Height);
            }
        }

        /// <summary>
        /// Constructors a new player.
        /// </summary>
        public Player(Level level, Vector2 position, int playerIdent)
        {
            this.level = level;
            PlayerIdent = playerIdent;
        }

        /// <summary>
        /// Loads the player sprite sheet and sounds.
        /// </summary>
        public void LoadContent()
        {
            otherPlayer = Level.Player[Math.Abs(PlayerIdent - 1)];

            // Load animated textures.
            if (this.PlayerIdent == 0)
            {
                idleAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player1_idle"), 0.1f, true);
                runAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player1_side"), 0.1f, false);
                jumpAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player1_jump"), 0.1f,
                                              false);
                jumprunAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player1_side_jump"),
                                                 0.1f, true);
            }
            else
            {
                idleAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player2_idle"), 0.1f, true);
                runAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player2_side"), 0.1f, false);
                jumpAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player2_jump"), 0.1f, true);
                jumprunAnimation = new Animation(Level.Content.Load<Texture2D>("scene/players/ch_player2_side_jump"),
                                                 0.1f, true);
            }
            //   celebrateAnimation = new Animation(Level.Content.Load<Texture2D>("Sprites/Player/Celebrate"), 0.1f, false);
            //   dieAnimation = new Animation(Level.Content.Load<Texture2D>("Sprites/Player/Die"), 0.1f, false);

            // Calculate bounds within texture size.            
            int width = (int) (idleAnimation.FrameWidth*0.4);
            int left = (idleAnimation.FrameWidth - width)/2;
            int height = (int) (idleAnimation.FrameWidth*0.8);
            int top = idleAnimation.FrameHeight - height;
            localBounds = new Rectangle(left, top, width, height);

            // TODO: add sound here
            // Load sounds.            
            //killedSound = Level.Content.Load<SoundEffect>("Sounds/PlayerKilled");
            jumpSound = Level.Content.Load<SoundEffect>("sounds/sprungfeder_cool_cartoon010");
            fallSound = Level.Content.Load<SoundEffect>("sounds/PlayerFall");
            gegnerSchiebenSound = Level.Content.Load<SoundEffect>("sounds/harter_aufschlag_cartoon011");
            gegnerZiehenSound = Level.Content.Load<SoundEffect>("sounds/quietsch_zipp_cartoon014");
            landeAufGegnerSound = Level.Content.Load<SoundEffect>("sounds/sprungfeder_doing_cartoon048");
        }

        /// <summary>
        /// Resets the player to life.
        /// </summary>
        /// <param name="position">The position to come to life at.</param>
        public void Reset(Vector2 position)
        {
            Position = position;
            Velocity = Vector2.Zero;
            isAlive = true;
            sprite.PlayAnimation(idleAnimation);
        }

        /// <summary>
        /// Handles input, performs physics, and animates the player sprite.
        /// </summary>
        public void Update(GameTime gameTime)
        {
            GetInput();

            ApplyPhysics(gameTime);

            if (IsAlive && IsOnGround)
            {
                if (Math.Abs(Velocity.X) - 0.02f > 0)
                {
                    sprite.PlayAnimation(runAnimation);
                }
                else
                {
                    sprite.PlayAnimation(idleAnimation);
                }
            }

            // Clear input.
            movement = 0.0f;
            isJumping = false;
        }

        /// <summary>
        /// Gets player horizontal movement and jump commands from input.
        /// </summary>
        /// 
        private void GetInput()
        {
            // Get input state.
            // GamePadState gamePadState = GamePad.GetState(PlayerIndex.One);
            KeyboardState keyboardState = Keyboard.GetState();

            // Get analog horizontal movement.
            // movement = gamePadState.ThumbSticks.Left.X * MoveStickScale;

            // Ignore small movements to prevent running in place.
            if (Math.Abs(movement) < 0.5f)
                movement = 0.0f;

            // If any digital horizontal movement input is found, override the analog movement.
            if (
                //gamePadState.IsButtonDown(Buttons.DPadLeft) ||
                (keyboardState.IsKeyDown(Keys.Left) && PlayerIdent == 0) ||
                (keyboardState.IsKeyDown(Keys.A) && PlayerIdent == 1))
            {
                movement = -1.0f;
            }
            else if (
                //gamePadState.IsButtonDown(Buttons.DPadRight) || // ignor 
                (keyboardState.IsKeyDown(Keys.Right) && PlayerIdent == 0) ||
                (keyboardState.IsKeyDown(Keys.D) && PlayerIdent == 1))
            {
                movement = 1.0f;
            }

            // Check if the player wants to jump.
            isJumping =
                // gamePadState.IsButtonDown(JumpButton) ||
                (keyboardState.IsKeyDown(Keys.Up) && PlayerIdent == 0) || // space input for test
                (keyboardState.IsKeyDown(Keys.W) && PlayerIdent == 1);
        }

        /// <summary>
        /// Updates the player's velocity and position based on input, gravity, etc.
        /// </summary>
        public void ApplyPhysics(GameTime gameTime)
        {
            float elapsed = (float) gameTime.ElapsedGameTime.TotalSeconds;
            Vector2 IntersektionVector;
            Vector2 previousPosition = Position;

            // Base velocity is a combination of horizontal movement control and
            // acceleration downward due to gravity.
            velocity.X += movement*MoveAcceleration*elapsed;
            velocity.Y = MathHelper.Clamp(velocity.Y + GravityAcceleration*elapsed, -MaxFallSpeed, MaxFallSpeed);

            velocity.Y = DoJump(velocity.Y, gameTime);

            // Apply pseudo-drag horizontally.
            if (IsOnGround)
                velocity.X *= GroundDragFactor;
            else
                velocity.X *= AirDragFactor;

            // Prevent the player from running faster than his top speed.            
            velocity.X = MathHelper.Clamp(velocity.X, -MaxMoveSpeed, MaxMoveSpeed);

            // Apply velocity.
            Position += velocity*elapsed;
            Position = new Vector2((float) Math.Round(Position.X), (float) Math.Round(Position.Y));
            IntersektionVector = new Vector2((float) 0.0f, (float) 0.0f);
            // If the player is now colliding with the level, separate them.
            IntersektionVector = HandleCollisions(IntersektionVector, previousPosition);
            // check collision  with player 1/2 and tiles

            // If the collision stopped us from moving, reset the velocity to zero.
            if (!CollisionVictim)
            {
                if (!CollisionReason)
                {
                    if (Position.X == previousPosition.X)
                        velocity.X = 0;

                    if (Position.Y == previousPosition.Y)
                        velocity.Y = 0;
                }
            }
            else
            {
                if (!CollisionReason)
                {
                    if (Position.X == previousPosition.X)
                        velocity.X = 0;

                    if (Position.Y == previousPosition.Y)
                        velocity.Y = 0;
                }
            }
        }

        /// <summary>
        /// Calculates the Y velocity accounting for jumping and
        /// animates accordingly.
        /// </summary>
        /// <remarks>
        /// During the accent of a jump, the Y velocity is completely
        /// overridden by a power curve. During the decent, gravity takes
        /// over. The jump velocity is controlled by the jumpTime field
        /// which measures time into the accent of the current jump.
        /// </remarks>
        /// <param name="velocityY">
        /// The player's current velocity along the Y axis.
        /// </param>
        /// <returns>
        /// A new Y velocity if beginning or continuing a jump.
        /// Otherwise, the existing Y velocity.
        /// </returns>
        private float DoJump(float velocityY, GameTime gameTime)
        {
            // If the player wants to jump
            if (isJumping)
            {
                // Begin or continue a jump
                if ((!wasJumping && IsOnGround) // replace for  "&& IsOnGround)" to test the acceleration
                    || jumpTime > 0.0f)
                {
                    if (jumpTime == 0.0f)
                        jumpSound.Play();
                    if (Velocity.X == 0)
                    {
                        jumpTime += (float) gameTime.ElapsedGameTime.TotalSeconds;
                        sprite.PlayAnimation(jumpAnimation);
                    }
                    else
                    {
                        jumpTime += (float) gameTime.ElapsedGameTime.TotalSeconds;
                        sprite.PlayAnimation(jumprunAnimation);
                    }
                }

                // If we are in the ascent of the jump
                if (0.0f < jumpTime && jumpTime <= MaxJumpTime)
                {
                    // Fully override the vertical velocity with a power curve that gives players more control over the top of the jump
                    velocityY = JumpLaunchVelocity*(1.0f - (float) Math.Pow(jumpTime/MaxJumpTime, JumpControlPower));
                }
                else
                {
                    // Reached the apex of the jump
                    jumpTime = 0.0f;
                }
            }
            else
            {
                // Continues not jumping or cancels a jump in progress
                jumpTime = 0.0f;
            }
            wasJumping = isJumping;

            return velocityY;
        }

        /// <summary>
        /// Detects and resolves all collisions between the player and his neighboring
        /// tiles. When a collision is detected, the player is pushed away along one
        /// axis to prevent overlapping. There is some special logic for the Y axis to
        /// handle platforms which behave differently depending on direction of movement.
        /// </summary>
        // add player collision Victimstate
        private Vector2 HandleCollisions(Vector2 intersectionDepth, Vector2 PrePosition)
        {
            // Get the player's bounding rectangle and find neighboring tiles.
            // HACK: look for player 2
            Rectangle bounds = BoundingRectangle;
            int leftTile = (int) Math.Floor((float) bounds.Left/Tile.Width);
            int rightTile = (int) Math.Ceiling(((float) bounds.Right/Tile.Width)) - 1;
            int topTile = (int) Math.Floor((float) bounds.Top/Tile.Height);
            int bottomTile = (int) Math.Ceiling(((float) bounds.Bottom/Tile.Height)) - 1;


            // Reset flag to search for ground collision.
            isOnGround = false;
            CollisionVictim = false;
            isLockedByChain = false;

            // For each potentially colliding tile,
            for (int y = topTile; y <= bottomTile; ++y)
            {
                for (int x = leftTile; x <= rightTile; ++x)
                {
                    // If this tile is collidable,
                    Tile.TileCollision collision = Level.GetCollision(x, y, PlayerIdent);
                    if (collision == Tile.TileCollision.ChainPlayer || collision == Tile.TileCollision.TouchPlayer)
                    {
                        if (collision == Tile.TileCollision.ChainPlayer)
                        {
                            if (isJumping)
                            {
                                if (Position.Y < otherPlayer.Position.Y)
                                {
                                    otherPlayer.velocity.X = velocity.X*0.4f;
                                    otherPlayer.velocity.Y = velocity.Y*0.4f;
                                }
                            }
                            Position = PrePosition; // deltaPos_vector2(PrePosition, Position);    

                            CollisionVictim = true;
                            velocity.X = velocity.X*-1.2f;
                            velocity.Y = velocity.Y*-1.2f;
                            // Perform further collisions with the new bounds.
                            bounds = BoundingRectangle;

                            isLockedByChain = true;
                        }
                        else
                        {
                            // if we are right on top of the other player allow jumping :)
                            if ((otherPlayer.Position.Y - Position.Y) < 130 &&
                                (otherPlayer.Position.Y - Position.Y) > 0)
                            {
                                if (Position.X > otherPlayer.Position.X - 25 && Position.X < otherPlayer.Position.X + 25)
                                {
                                    isOnGround = true;
                                }
                            }

                            // presetposition to the wanted and reduce the position about the aktuell delta 
                            Position = PrePosition; //  deltaPos_vector2(PrePosition, Position);
                            velocity.X = velocity.X*-1.0f;
                            velocity.Y = velocity.Y*-1.0f;
                            CollisionVictim = false;
                            // Perform further collisions with the new bounds.
                            bounds = BoundingRectangle;
                        }
                    }

                    if (collision != Tile.TileCollision.Passable)
                    {
                        // Determine collision depth (with direction) and magnitude.
                        Rectangle tileBounds = Level.GetBounds(x, y);
                        Vector2 depth = RectangleExtensions.GetIntersectionDepth(bounds, tileBounds);
                        if (depth != Vector2.Zero)
                        {
                            float absDepthX = Math.Abs(depth.X);
                            float absDepthY = Math.Abs(depth.Y);

                            // Resolve the collision along the shallow axis.
                            if (absDepthY < absDepthX || collision == Tile.TileCollision.Platform)
                            {
                                // If we crossed the top of a tile, we are on the ground.
                                if (previousBottom <= tileBounds.Top && collision == Tile.TileCollision.Platform)
                                    isOnGround = true;

                                // Ignore platforms, unless we are on the ground.
                                if (collision == Tile.TileCollision.Impassable || IsOnGround)
                                {
                                    // Resolve the collision along the Y axis.
                                    Position = new Vector2(Position.X, Position.Y + depth.Y);

                                    // when we hit a platform from below
                                    if (isJumping)
                                    {
                                        velocity.Y = 0f;
                                        jumpTime = MaxJumpTime;
                                    }

                                    // Perform further collisions with the new bounds.
                                    bounds = BoundingRectangle;
                                }
                            }
                            else if (collision == Tile.TileCollision.Impassable) // Ignore platforms.
                            {
                                // Resolve the collision along the X axis.
                                Position = new Vector2(Position.X + depth.X, Position.Y);

                                // Perform further collisions with the new bounds.
                                bounds = BoundingRectangle;
                            }
                        }
                    }
                }
            }
            return intersectionDepth;
        }

        /// <summary>
        /// Called when the player has been killed.
        /// </summary>
        /// <param name="killedBy">
        /// The enemy who killed the player. This parameter is null if the player was
        /// not killed by an enemy (fell into a hole).
        /// </param>
        /*public void OnKilled(Enemy killedBy)
        {
            isAlive = false;

            if (killedBy != null)
                killedSound.Play();
            else
                fallSound.Play();

            sprite.PlayAnimation(dieAnimation);
        }*/
        /// <summary>
        /// Called when this player reaches the level's exit.
        /// </summary>
        public void OnReachedExit()
        {
            // sprite.PlayAnimation(celebrateAnimation);
        }

        /// <summary>
        /// Draws the animated player.
        /// </summary>
        public void Draw(GameTime gameTime, SpriteBatch spriteBatch, Vector2 cameraPosition)
        {
            // Flip the sprite to face the way we are moving.
            if (Velocity.X < 0) // wurde umgedreht, vorheriges Ikon war umgedreht orientiert
                flip = SpriteEffects.FlipHorizontally;
            else if (Velocity.X > 0)
                flip = SpriteEffects.None;

            // Draw that sprite.
            sprite.Draw(gameTime, spriteBatch, new Vector2(Position.X, Position.Y - (cameraPosition.Y - 384 - 128)),
                        flip);
        }

        private float get_checkvectormax(float reference, float candidat)
        {
            if (reference > 0.0f)
            {
                if (candidat > reference)
                    return candidat;
                else
                    return reference;
            }
            else
            {
                if (candidat < reference)
                    return candidat;
                else
                    return reference;
            }
        }

        private Vector2 deltaPos_vector2(Vector2 PrePosition, Vector2 Position)
        {
            float dx;
            float dy;

            // generate deviation
            dx = (Position.X - PrePosition.X)*0.3f;
            if ((int) Math.Abs(dx) < 20)
                dx = dx*2.0f;

            dy = (Position.Y - PrePosition.Y)*0.3f;
            if ((int) Math.Abs(dy) < 20)
                dy = dy*2.0f;

            Position.X = Position.X + dx;
            Position.Y = Position.Y + dy;

            return Position;
        }
    }
}