﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace pipa.Scene
{


    public class Tile
    {
        private TileCollision collisionType;
        public TileCollision Collision
        {
            get { return collisionType; }
        }

        private int idx;
        public static int Width = 64;
        public static int Height = 48;

        public int Index
        {
            get { return idx;}
        }

        public Tile(int tileIndex) : this(tileIndex, TileCollision.Passable)
        {

        }

        public Tile(int tileIndex, TileCollision collisionType)
        {
            this.idx = tileIndex;
            this.collisionType = collisionType;
        }

        /// <summary>
        /// Controls the collision detection and response behavior of a tile.
        /// </summary>
        public enum TileCollision
        {
            /// <summary>
            /// A passable tile is one which does not hinder player motion at all.
            /// </summary>
            Passable = 0,

            /// <summary>
            /// An impassable tile is one which does not allow the player to move through
            /// it at all. It is completely solid.
            /// </summary>
            Impassable = 1,

            /// <summary>
            /// A platform tile is one which behaves like a passable tile except when the
            /// player is above it. A player can jump up through a platform as well as move
            /// past it to the left and right, but can not fall down through the top of it.
            /// </summary>
            Platform = 2,

            /// <summary>
            /// get in touch with the other player 
            /// </summary>
            TouchPlayer = 3,

            /// <summary>
            /// keep in touch with the other player max length
            /// </summary>
            ChainPlayer = 4,
        }
    }
}
